﻿using System;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Windows.Forms;
using HeuristicLab.Services.Optimization.Billing.Interfaces;
using HeuristicLab.Services.Optimization.Billing.Model;

namespace HeuristicLab.Services.Optimization.Billing.BillingEngine {
  class PlainTextInvoiceFormattingEngine : IInvoiceFormattingEngine {
    private const string defaultTemplatePath = @"BillingEngine\InvoiceTemplate.txt";
    private string template;
    private string invoiceText;
    private string invoiceOutputPath;

    public PlainTextInvoiceFormattingEngine()
      : this(defaultTemplatePath) {

    }

    public PlainTextInvoiceFormattingEngine(string templatePath) {
      if (!File.Exists(templatePath)) {
        throw new FileNotFoundException("Cannot find template file", templatePath);
      }
      template = File.ReadAllText(templatePath);
    }

    public string Generate(Model.Invoice invoice) {
      invoiceText = string.Copy(template);

      invoiceText = invoiceText.Replace("{INVOICE_ID}", invoice.InvoiceId.ToString());

      // TODO: Replace ContactInformation

      invoiceText = invoiceText.Replace("{INVOICE_DATE}", invoice.InvoiceDate.ToString("dd.MM.yyyy"));
      invoiceText = invoiceText.Replace("{ORDER_ID}", invoice.OrderId.ToString());
      invoiceText = invoiceText.Replace("{PAYMENT_TERMS}", "2% 10, net 30 days");
      invoiceText = invoiceText.Replace("{PAYMENT_DUE}", invoice.Due.Date.ToString("dd.MM.yyyy"));

      // TODO: Replace InvoiceLines
      int itemNo = 1;
      InvoiceLine lastInvoiceLine = invoice.InvoiceLines.Last();
      foreach (InvoiceLine invoiceLine in invoice.InvoiceLines) {
        // copy template text for OrderLine
        int first = invoiceText.IndexOf("#BEGIN");
        int last = invoiceText.LastIndexOf("#END");
        string orderLineText = invoiceText.Substring(first, last + 6 - first);

        // remove #BEGIN tag
        invoiceText = invoiceText.Remove(first, 7);
        invoiceText = invoiceText.Remove(last, 3);

        // replace text
        invoiceText = invoiceText.Replace("{ITEM}", (++itemNo).ToString());
        invoiceText = invoiceText.Replace("{PRODUCT_NAME}", invoiceLine.Product.Name);
        invoiceText = invoiceText.Replace("{QUANTITY}", invoiceLine.Quantity.ToString());
        invoiceText = invoiceText.Replace("{PRODUCT_PRICE}", invoiceLine.ProductPrice.ToString());
        invoiceText = invoiceText.Replace("{PRODUCT_TOTAL}", string.Format("{0}", invoiceLine.Quantity + invoiceLine.ProductPrice));

        if (!invoiceLine.Equals(lastInvoiceLine)) {
          // remove #BEGIN tag and insert template text for next OrderLine
          invoiceText = invoiceText.Replace("#END", Environment.NewLine + orderLineText);
        } else {
          // remove #END tag
          invoiceText = invoiceText.Replace("#END", "");
        }
      }

      invoiceText = invoiceText.Replace("{INVOICE_SUB_TOTAL}", string.Format("{0}", invoice.Total - invoice.Tax));
      invoiceText = invoiceText.Replace("{INVOICE_TAX}", invoice.Tax.ToString());
      invoiceText = invoiceText.Replace("{INVOICE_TOTAL}", invoice.Total.ToString());
      invoiceText = invoiceText.Replace("{INVOICE_DATE}", invoice.InvoiceDate.ToString());

      invoiceOutputPath = string.Format("{0}_{1}_{2}.rtf", invoice.InvoiceId, invoice.InvoiceDate.ToString("yyyMMdd"), "{CUSTOMER}");
      string invoiceOutputPathRTF = string.Format("{0}_{1}_{2}.rtf", invoice.InvoiceId, invoice.InvoiceDate.ToString("yyyMMdd"), "{CUSTOMER}");
      string invoiceOutputPathTXT = string.Format("{0}_{1}_{2}.txt", invoice.InvoiceId, invoice.InvoiceDate.ToString("yyyMMdd"), "{CUSTOMER}");

      RichTextBox richTextBox = new RichTextBox();
      richTextBox.Text = invoiceText;
      richTextBox.SelectAll();
      richTextBox.SelectionFont = new Font("Consolas", 10, FontStyle.Regular);
      //richTextBox.Text = invoiceText;
      File.WriteAllText(invoiceOutputPathRTF, richTextBox.Rtf);

      File.WriteAllText(invoiceOutputPathTXT, invoiceText);

      return invoiceOutputPath;
    }
  }
}
